"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MikroFlusherInterceptor = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@mikro-orm/core");
const common_1 = require("@nestjs/common");
const graphql_1 = require("@nestjs/graphql");
const rxjs_1 = require("rxjs");
const mikro_flusher_context_class_1 = require("./mikro-flusher-context.class");
/**
 * Invoke `em.flush()` after all mutations have been handled.
 */
let MikroFlusherInterceptor = class MikroFlusherInterceptor {
    constructor(em) {
        this.em = em;
    }
    intercept(executionContext, next) {
        const operation = graphql_1.GqlExecutionContext.create(executionContext).getInfo()
            .operation.operation;
        if (operation != "mutation")
            return next.handle();
        const context = mikro_flusher_context_class_1.MikroFlusherContext.current;
        if (!context)
            throw new Error("The middleware must be applied for this route");
        context.mutationCountTotal++;
        return next.handle().pipe((0, rxjs_1.concatMap)(async (result) => {
            context.mutationCountHandled++;
            const flush = context.mutationCountHandled == context.mutationCountTotal
                ? this.em
                    .flush()
                    .then(() => context.flush.resolve())
                    .catch((err) => context.flush.reject(err))
                : context.flush;
            await flush;
            return result;
        }));
    }
};
MikroFlusherInterceptor = (0, tslib_1.__decorate)([
    (0, common_1.Injectable)(),
    (0, tslib_1.__metadata)("design:paramtypes", [core_1.EntityManager])
], MikroFlusherInterceptor);
exports.MikroFlusherInterceptor = MikroFlusherInterceptor;
//# sourceMappingURL=mikro-flusher.interceptor.js.map